@@ -1,12 +1,19 @@
-;;; preview-latex.el --- automatically extracted autoloads
+;;; preview-latex.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "preview" "preview.el" (25122 39049 217449
+;;;### (autoloads nil "preview" "preview.el" (25679 44930 923593
 ;;;;;;  100000))
 ;;; Generated autoloads from preview.el
 
+(autoload 'desktop-buffer-preview "preview" "\
+Hook function for restoring persistent previews into a buffer.
+
+\(fn FILE-NAME BUFFER-NAME MISC)" nil nil)
+
+(add-to-list 'desktop-buffer-mode-handlers '(latex-mode . desktop-buffer-preview))
+
 (autoload 'preview-install-styles "preview" "\
 Installs the TeX style files into a permanent location.
 This must be in the TeX search path.  If FORCE-OVERWRITE is greater
@@ -28,7 +35,7 @@ to add the preview functionality." nil nil)
 (autoload 'preview-report-bug "preview" "\
 Report a bug in the preview-latex package." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "preview" '("TeX-" "desktop-buffer-preview" "preview-")))
+(register-definition-prefixes "preview" '("TeX-" "desktop-buffer-preview-misc-data" "preview-"))
 
 ;;;***
 
